<?php

namespace Drupal\dea_blocker\Classes;

use \Drupal\Core\Config\Config;
use \Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Manages a blacklist of email addresses and domains.
 * Provides methods to test email against that list.
 *
 * @author Claudio Nicora <coolsoft.ita@gmail.com>
 */
class EmailBlacklist {

  /**
   * Configuration object.
   * @var Config
   */
  protected $settings = NULL;


  /**
   * Service constructor.
   * @param ConfigFactoryInterface $configFactory
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->settings = $configFactory->get('dea_blocker.settings');
  }


  /**
   * Extract mail domain from the given email address.
   * The result is lower-cased.
   *
   * @param string $email
   *   The email address.
   *
   * @return string|FALSE
   *   Mail domain or FALSE in case of malformed email address.
   */
  static public function getDomain(string $email) {
    $domain = [];
    if (preg_match('/@(.*)$/', $email, $domain) !== 0) {
      return strtolower($domain[1]);
    }
    else {
      return FALSE;
    }
  }


  /**
   * Returns TRUE if the email address is blacklisted, FALSE otherwise.
   *
   * @param string $email
   *   The email address to test.
   *
   * @return string|FALSE
   *   If the email address is blacklisted returns the found pattern,
   *   FALSE otherwise.
   */
  public function isBlacklisted(string $email) {

    $email = strtolower(trim($email));

    // Extract mail domain and test it against domain blacklist.
    $domain = self::getDomain($email);
    if ($domain) {

      // Test if domain or subdomain is blacklisted.
      $blacklist = $this->settings->get('blacklist') ?: [];
      foreach ($blacklist as $item) {
        // Test if value ends with the blacklist item.
        if (substr($domain, -strlen($item)) == $item) {
          return $item;
        }
      }

      // Test domain against regular expressions blacklist
      $blacklist_re = $this->settings->get('blacklist_re') ?: [];
      foreach ($blacklist_re as $pattern) {
        if (preg_match($pattern, $domain)) {
          return $item;
        }
      }

    }

    // Not blacklisted.
    return FALSE;

  }

}
