<?php

namespace Drupal\dea_blocker\Form;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\dea_blocker\Classes\EmailBlacklist;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Configure module.
 */
class AdminForm extends ConfigFormBase {

  /**
   * @var Config
   */
  protected $settings;

  /**
   * @var Drupal\user\PrivateTempStore
   */
  protected $tempStore;


  public function __construct(ConfigFactoryInterface $config_factory, \Drupal\user\PrivateTempStoreFactory $tempStoreFactory) {
    parent::__construct($config_factory);
    $this->settings = $this->config('dea_blocker.settings');
    $this->tempStore = $tempStoreFactory->get('dea_blocker.admin');
  }


  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('user.private_tempstore')
    );
  }


  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dea_blocker_admin';
  }


  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [ 'dea_blocker.settings' ];
  }


  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Save new default tab.
    if (array_key_exists('settings__active_tab', $form_state->getUserInput())) {
      $this->tempStore->set('active_tab', $form_state->getUserInput()['settings__active_tab']);
    }

    // Title & styles.
    $form['#title'] = t('DEA blocker settings');

    // Main tab container.
    $form['settings'] = [
      '#type' => 'vertical_tabs',
      '#default_tab' => $this->tempStore->get('active_tab'),
    ];

    $form['forms'] = [
      '#type' => 'details',
      '#title' => 'Forms',
      '#group' => 'settings',
    ] + $this->buildFormsTab($form, $form_state);

    $form['blacklists'] = [
      '#type' => 'details',
      '#title' => 'Blacklists',
      '#group' => 'settings',
    ] + $this->buildBlacklistTab($form, $form_state);

    $form['test'] = [
      '#type' => 'details',
      '#title' => 'Test email',
      '#group' => 'settings',
    ] + $this->buildTestTab($form, $form_state);

    return parent::buildForm($form, $form_state);

  }


  /**
   * Build and return "Forms" configuration tab.
   *
   * @return array
   *   Renderable form array.
   */
  protected function buildFormsTab(array $form, FormStateInterface $form_state) {

    $forms = $this->settings->get('forms') ?: [];

    $options = [
      'all'       => t('all forms'),
      '!sel' => t('all forms except the ones listed below'),
      'sel'  => t('only forms listed below'),
      'off'       => t('none (off)'),
    ];
    $elem['mode'] = [
      '#type' => 'radios',
      '#title' => t('Protect email fields on...'),
      '#options' => $options,
      '#default_value' => $this->settings->get('mode') ?: 'off',
    ];

    $elem['forms'] = [
      '#type' => 'textarea',
      '#description' => t('List of form IDs, one per line.'),
      '#default_value' => $forms
                          ? implode("\r\n", $forms) . "\r\n"
                          : '',
      '#states' => [
        'enabled' => [
          [':input[name="mode"]' => ['value' => 'sel']],
          'or',
          [':input[name="mode"]' => ['value' => '!sel']],
        ],
      ],
    ];

    $elem['show_form_selector'] = [
      '#type' => 'checkbox',
      '#title' => t('Append additional block under each form to ease form ID selection'),
      '#description' => t('NOTE: block will be shown only to users with "Administer DEA blocker" permission.'),
      '#default_value' => $this->settings->get('show_form_selector'),
    ];

    return $elem;

  }


  /**
   * Build and return "Blacklist" configuration tab.
   *
   * @return array
   *   Renderable form array.
   */
  protected function buildBlacklistTab(array $form, FormStateInterface $form_state) {

    $blacklist = $this->settings->get('blacklist') ?: [];
    $elem['blacklist'] = [
      '#type' => 'textarea',
      '#title' => t('Domain blacklist'),
      '#description' => t(
        'Insert blacklisted mail domains, one per line (list is automatically deduplicated and sorted).<br/><b>NOTE:</b> will match subdomains too, so adding @domain will blacklist both @email1 and @email2.', [
        '@domain' => Markup::create('<b>example.org</b>'),
        '@email1' => Markup::create('<b>foo@example.org</b>'),
        '@email2' => Markup::create('<b>bar@subdomain.example.org</b>'),
      ]),
      '#default_value' => $blacklist ? implode("\r\n", $blacklist) ."\r\n" : '',
      '#rows' => 10,
    ];

    $blacklist_re = $this->settings->get('blacklist_re') ?: [];
    $elem['blacklist_re'] = [
      '#type' => 'textarea',
      '#title' => t('Domain regular expression blacklist'),
      '#description' => t('Insert blacklisted mail domains regular expressions, one per line.<br/><b>NOTE:</b> each line must be a valid @preg_match_link regular expression that will be applied to domain name, like @example.', [
        '@preg_match_link' => Markup::create('<a href="http://php.net/manual/function.preg-match.php">preg_match()</a>'),
        '@example' => Markup::create('<b>/^example\.(org|net|com)$/</b>'),
      ]),
      '#default_value' => $blacklist_re
                          ? implode("\r\n", $blacklist_re) . "\r\n"
                          : '',
      '#rows' => 10,
    ];

    return $elem;

  }


  /**
   * Build and return "Test" configuration tab.
   *
   * @return array
   *   Renderable form array.
   */
  protected function buildTestTab(array $form, FormStateInterface $form_state) {

    $elem['test_email'] = [
      '#type' => 'email',
      '#title' => t('Test email'),
      '#description' => t('Test this email address against configured blacklists'),
      '#default_value' => $this->tempStore->get('test_email'),
    ];
    $elem['actions'] = [
      '#type' => 'actions',
      '#value' => t('Test'),
    ];
    $elem['actions']['cmdTest'] = [
      '#type' => 'submit',
      '#value' => t('Test'),
      '#submit' => ['::cmdTest'],
    ];
    return $elem;

  }


  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    // Save blacklist.
    $form_state->setValue('blacklist', $this->stringListToArray($form_state->getValue('blacklist', '')));

    // Save regex blacklist (checking regex validity).
    $blacklist_re = $this->stringListToArray($form_state->getValue('blacklist_re', ''));
    $re_errors = $this->testRegExpPatterns($blacklist_re);
    if ($re_errors) {
      $error_msg = t('One or more regular expression patterns are not valid:');
      foreach ($re_errors as $pattern => $pattern_error) {
        $error_msg .= '<br/><b>'.$pattern.'</b> - '.$pattern_error;
      }
      $form_state->setErrorByName('blacklist_re', Markup::create($error_msg));
    }
    else {
      $form_state->setValue('blacklist_re', $blacklist_re);
    }

    // Form IDs.
    $form_state->setValue('forms', $this->stringListToArray($form_state->getValue('forms', '')));

    parent::validateForm($form, $form_state);

  }


  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Save field values.
    $fields = [
      'blacklist',
      'blacklist_re',
      'show_form_selector',
      'mode',
      'forms',
    ];
    foreach ($fields as $field) {
      $this->settings->set($field, $form_state->getValue($field));
    }
    $this->settings->save();

    parent::submitForm($form, $form_state);

  }


  /**
   * Test the email against blacklist.
   */
  public function cmdtest(array &$form, FormStateInterface $form_state) {

    $email = $form_state->getValue('test_email', '');
    $this->tempStore->set('test_email', $email);

    if ($email) {
      // Blacklist service instance.
      /* @var $blacklist EmailBlacklist */
      $blacklist = \Drupal::getContainer()->get('dea_blocker.emailblacklist');

      if ($blacklist->isBlacklisted($email)) {
        drupal_set_message(
          t('Email address :email is blacklisted', [':email' => $email]),
          'error'
        );
      }
      else {
        drupal_set_message(
          t('Email address :email is not blacklisted', [':email' => $email])
        );
      }
    }
    else {
      drupal_set_message(
        t('Please insert an email value to test.'),
        'error'
      );
    }

  }


  /**
   * Transform the given items list (string), separated with \r\n, into an array:
   * - converted to lowercase
   * - empty lines removed
   * - duplicate lines removed
   * - sorted
   *
   * @param string $value
   *   Items string, separated by \r\n.
   *
   * @return array
   */
  protected function stringListToArray(string $value) {
    return $this->cleanupArray(explode("\r\n", strtolower($value)));
  }


  /**
   * Cleanup the given array :
   * - empty items removed
   * - duplicate items removed
   * - sorted
   *
   * @param array $value
   *   Array to cleanup.
   *
   * @return array
   */
  protected function cleanupArray(array $value) {
    $array = array_filter(array_unique($value));
    sort($array);
    return $array;
  }


  /**
   * Test the given regular expression patterns for validity.
   *
   * @param string[] $regexps
   *   Regular expression patterns to test.
   *
   * @return array
   *   Array with bad regular expression patterns, indexed by pattern.
   */
  protected function testRegExpPatterns(array $regexps) {

    // Set a local error handler.
    set_error_handler(function ($errno, $errstr, $errfile, $errline) {
      throw new \ErrorException($errstr);
    });

    // Test patterns.
    $errors = [];
    foreach ($regexps as $regexp) {
      try {
        preg_match($regexp, '');
      } catch (\ErrorException $ex) {
        $errors[$regexp] = $ex->getMessage();
      }
    }

    // Restore previous error handler.
    restore_error_handler();
    return $errors;
  }


  /**
   * Add or remove a FormID to/from the forms list.
   * This method is called in a controller-like way.
   *
   * @param string $command
   *   Command to execute, can be 'add' or 'remove'.
   * @param string $formId
   *   ID of the form to add/remove.
   */
  public function addRemoveForm($command, $formId) {

    if (!$formId) return;

    $forms = $this->settings->get('forms');
    switch ($command) {
      case 'add_form':
        $forms[] = $formId;
        drupal_set_message(t('Form :formId added to list', [':formId' => $formId]));
        break;
      case 'remove_form':
        if (($key = array_search($formId, $forms)) !== FALSE) {
          unset($forms[$key]);
        }
        drupal_set_message(t('Form :formId removed from list', [':formId' => $formId]));
        break;
    }
    $this->settings
      ->set('forms', $this->cleanupArray($forms))
      ->save();

    // Set the default tab.
    $this->tempStore->set('active_tab', 'edit-forms');

    // Returns a RedirectResponse.
    return new RedirectResponse(Url::fromRoute('dea_blocker.admin')->toString());

  }


}
